package com.yizhi.calendar.application.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.yizhi.calendar.application.task.CalendarAccountExportAsync;
import com.yizhi.calendar.application.task.CalendarDetailExportAsync;
import com.yizhi.calendar.application.task.CalendarListExportAsync;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.plugins.pagination.PageHelper;
import com.yizhi.calendar.application.service.ICalendarRecordService;
import com.yizhi.calendar.application.vo.CalendarChartVO;
import com.yizhi.calendar.application.vo.CalendarStatisticVO;

@RestController
@RequestMapping("/statistic/calendar")
public class CalendarStatisticController {

	@Autowired
	private ICalendarRecordService calendarRecordService;
	
	@Autowired
	private CalendarListExportAsync calendarListExportAsync;
	
	@Autowired
	private CalendarAccountExportAsync calendarAccountExportAsync;
	
	@Autowired
	private CalendarDetailExportAsync calendarDetailExportAsync;
	
	@Autowired
	private AccountClient accountClient;

	@GetMapping("/chart")
	public List<CalendarChartVO> signChart(@RequestParam(name = "startDate", required = true) String startDate,
			@RequestParam(name = "endDate", required = true) String endDate) {
		RequestContext res = ContextHolder.get();
		Long companyId = res.getCompanyId();
		Long siteId = res.getSiteId();
		List<CalendarChartVO> list = calendarRecordService.signChart(startDate, endDate, companyId, siteId);
		return list;
	}

	@GetMapping("/list")
	public Page<CalendarStatisticVO> list(@RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "kwd", required = false) String kwd,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo) {
		RequestContext res = ContextHolder.get();
		Page<CalendarStatisticVO> page = new Page<CalendarStatisticVO>();
		Long companyId = res.getCompanyId();
		Long siteId = res.getSiteId();
        if(pageSize != null && pageNo != null){
            PageHelper.startPage(pageNo,pageSize);
        }
		List<CalendarStatisticVO> list = calendarRecordService.list(startDate, endDate, kwd, companyId, siteId);
        if(pageSize != null && pageNo != null){

        	page.setTotal(PageHelper.getTotal());
        	page.setSize(pageSize);
        	page.setCurrent(pageNo);
            PageHelper.remove();
        }
		page.setRecords(list);
		return page;
	}
	
	@GetMapping("/list/export")
	public Map<String, Object> listExport(@RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,
            @RequestParam(name = "kwd", required = false) String kwd){
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "打卡统计-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("kwd", kwd);
		map.put("startDate", startDate);
		map.put("endDate", endDate);
		map.put("result", result);
		calendarListExportAsync.execute(map, true);
		return map;
	}
	
	@GetMapping("/account/export")
	public Map<String, Object> accountExport(@RequestParam(name = "startDate", required = true) String startDate,
            @RequestParam(name = "endDate", required = true) String endDate,@RequestParam(name = "userId", required = true) Long userId){
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		AccountVO accountVO = accountClient.findById(userId);
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = accountVO.getName()+"打卡记录-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("startDate", startDate);
		map.put("endDate", endDate);
		map.put("userId", userId);
		map.put("accountName", accountVO.getName());
		map.put("result", result);
		calendarAccountExportAsync.execute(map, true);
		return map;
	}
	
	@GetMapping("/detail/export")
	public Map<String, Object> detailExport(@RequestParam(name = "startDate", required = true) String startDate,
			@RequestParam(name = "endDate", required = true) String endDate,
			@RequestParam(name = "kwd", required = false) String kwd){
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "打卡记录-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("kwd", kwd);
		map.put("startDate", startDate);
		map.put("endDate", endDate);
		map.put("result", result);
		calendarDetailExportAsync.execute(map, true);
		return map;
		
	}
}
