package com.yizhi.calendar.application.service.impl;

import cn.hutool.core.codec.Base64Encoder;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.plugins.pagination.PageHelper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.calendar.application.domain.CalendarPointSet;
import com.yizhi.calendar.application.domain.CalendarPoster;
import com.yizhi.calendar.application.domain.CalendarRecord;
import com.yizhi.calendar.application.mapper.CalendarPosterMapper;
import com.yizhi.calendar.application.mapper.CalendarRecordMapper;
import com.yizhi.calendar.application.service.ICalendarRecordService;
import com.yizhi.calendar.application.vo.*;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import com.yizhi.util.application.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 日历打卡记录表 服务实现类
 * </p>
 *
 * @author fulan123
 * @since 2019-10-10
 */
@Service
public class CalendarRecordServiceImpl extends ServiceImpl<CalendarRecordMapper, CalendarRecord>
		implements ICalendarRecordService {

	private static final Logger LOG = LoggerFactory.getLogger(CalendarRecordServiceImpl.class);

	@Autowired
	private CalendarRecordMapper calendarRecordMapper;

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private AccountClient accountClient;

	@Autowired
	private ReportClient reportClient;

	@Autowired
	private OrganizationClient organizationClient;

	@Autowired
	private CalendarPosterMapper calendarPosterMapper;

	@Override
	public Map<String, Object> sign(Long accountId, Long siteId, com.yizhi.calendar.application.domain.Calendar calendar, List<CalendarPointSet> list) {
		Map<String, Object> resultMap = new HashMap<String, Object>();
		RequestContext res = ContextHolder.get();
		Date date = new Date();
		String dayString = DateUtil.toDay(date);
		String timeString = DateUtil.toShortSeconds(date);
		CalendarSignViewVO signViewVO = this.isSign(accountId, siteId, calendar, list);
		if (null != signViewVO) {
			Integer isSign = signViewVO.getIsSign();
			Long day = signViewVO.getDay();
			Integer point = signViewVO.getPoint();
			if (1 == isSign) {
				resultMap.put("result", "签到失败");
				LOG.error("用户:" + accountId + "今日已签到，不能重复签到");
			} else {
				CalendarRecord record = new CalendarRecord();
				record.setId(idGenerator.generate());
				record.setAccountId(accountId);
				record.setCompanyId(res.getCompanyId());
				record.setSiteId(siteId);
				record.setOrgId(res.getOrgId());
				record.setCreateTime(date);
				record.setOrgId(res.getOrgId());
				record.setSignDate(dayString);
				record.setSignTime(timeString);
				record.setPoint(point);
				record.setPosterId(signViewVO.getPosterId());
				if (day > 0) {
					record.setFirstSign(0);
				} else {
					record.setFirstSign(1);
				}
				boolean b = this.insert(record);
				if (b) {
					HeadVO sumDayAndPoint = calendarRecordMapper.getSumDayAndPoint(accountId, siteId);
					resultMap.put("result", "签到成功");
					resultMap.put("consecutiveDays", signViewVO.getDay()+1);
					resultMap.put("sumDays", sumDayAndPoint.getSumDays());
					resultMap.put("point", point);
					resultMap.put("signId", record.getId());
					LOG.info("用户：" + accountId + "签到成功！  奖励积分：" + point);
				} else {
					resultMap.put("result", "签到失败");
					LOG.error("用户：" + accountId + "签到入库失败");
				}
			}
		} else {
			resultMap.put("result", "签到失败");
		}
		return resultMap;
	}

	@Override
	public CalendarSignViewVO isSign(Long accountId, Long siteId, com.yizhi.calendar.application.domain.Calendar calendar, List<CalendarPointSet> list) {
		CalendarSignViewVO vo = new CalendarSignViewVO();
		SimpleDateFormat format = new SimpleDateFormat("yyyymmddHHmmss");
		int i = java.util.Calendar.getInstance().get(java.util.Calendar.DAY_OF_MONTH);
		Date date = new Date();
		String dayString = DateUtil.toDay(date);
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-mm-dd");
		AccountVO accountVO = accountClient.findById(accountId);
		HeadVO headData = calendarRecordMapper.getSumDayAndPoint(accountId, siteId);
		List<CalendarRecord> lastFirstSigns = calendarRecordMapper.getLastFirstSignByaccountId(accountId, siteId);
		CalendarRecord record = new CalendarRecord();
		record.setSignDate(dayString);
		record.setAccountId(accountId);
		record.setSiteId(siteId);
		EntityWrapper<CalendarRecord> wrapper = new EntityWrapper<CalendarRecord>(record);
		CalendarRecord one = this.selectOne(wrapper);
		try {
			Date day = sdf.parse(dayString);
			if (CollectionUtils.isNotEmpty(lastFirstSigns)) {
				CalendarRecord lastFirstSign = lastFirstSigns.get(0);
				Date lastFirstSignDay = sdf.parse(lastFirstSign.getSignDate());
				long firstDayBetween = (day.getTime() - lastFirstSignDay.getTime()) / (60 * 60 * 24 * 1000);
				LOG.info("距离第一次签到"+firstDayBetween+"天");
				// 已签到
				if (!ObjectUtils.isEmpty(one)) {
					vo.setName(null == accountVO.getFullName() || "" == accountVO.getFullName() ? accountVO.getName()
							: accountVO.getFullName());
					vo.setHeadPortrait(!StringUtils.isEmpty(accountVO.getHeadPortrait())?imageToBase64(accountVO.getHeadPortrait()):"");
					vo.setDay(firstDayBetween + 1);
					vo.setIsSign(1);
					vo.setPoint(one.getPoint());
					vo.setIsCycle(0 != (vo.getDay() % calendar.getCycle()) ? 0 : 1);
					vo.setSumDays(headData.getSumDays());
					CalendarPoster poster = calendarPosterMapper.selectById(one.getPosterId());
					if (null != poster) {
						vo.setPosterId(poster.getId());
						vo.setImgUrl(poster.getBase64Code());
						vo.setContent(poster.getContent());
					}
					// 未签到
				} else {
					List<CalendarRecord> lastSigns = calendarRecordMapper.getLastSignByaccountId(accountId, siteId);
					CalendarRecord lastSign = lastSigns.get(0);
					Date lastSignDay = sdf.parse(lastSign.getSignDate());
					if (lastSign.getPosterId()==null) {
						lastSign.setPosterId((long) (new Random().nextInt(21)+1));
					}
					long dayBetween = (day.getTime() - lastSignDay.getTime()) / (60 * 60 * 24 * 1000);

					if (calendar.getUpdateTime().after(lastSign.getCreateTime())) {
						vo.setName(null == accountVO.getFullName() || "" == accountVO.getFullName()
								? accountVO.getName()
								: accountVO.getFullName());
						vo.setHeadPortrait(!StringUtils.isEmpty(accountVO.getHeadPortrait())?imageToBase64(accountVO.getHeadPortrait()):"");
						vo.setDay(0l);
						vo.setIsSign(0);
						vo.setIsCycle(0 != ((vo.getDay() + 1) % calendar.getCycle()) ? 0 : 1);
						vo.setPoint(calendar.getFirstPoint());
						vo.setSumDays(headData.getSumDays());
						CalendarPoster poster = calendarPosterMapper.selectById(vo.getIsCycle().equals(0)?((long)new Random().nextInt(21)+1):((long)new Random().nextInt(10)+22));
						if (null != poster) {
							vo.setPosterId(poster.getId());
							vo.setImgUrl(poster.getBase64Code());
							vo.setContent(poster.getContent());
						}
					} else {
						// 连续签到
						if (1 == dayBetween) {
							// 周期内任意一天
							if (0 != (firstDayBetween % calendar.getCycle())) {
								// 连续递增
								Integer point = 0;
								if (calendar.getSetType() == 1) {
									point = lastSign.getPoint() + list.get(0).getPoint();
									// 自定义
								} else {
									List<CalendarRuleListVO> ruleList = new ArrayList<CalendarRuleListVO>();
									// 组装自定义范围及积分
									for (CalendarPointSet pointSet : list) {
										Integer maxDate = calendarRecordMapper.getMaxDate(pointSet.getCalendarId(),
												pointSet.getMinDate());
										CalendarRuleListVO ruleListVO = new CalendarRuleListVO();
										ruleListVO.setMaxDate(maxDate);
										ruleListVO.setMinDate(pointSet.getMinDate());
										ruleListVO.setPoint(pointSet.getPoint());
										ruleList.add(ruleListVO);
									}
									if (CollectionUtils.isNotEmpty(ruleList)) {
										for (CalendarRuleListVO calendarRuleListVO : ruleList) {
											if (firstDayBetween >= calendarRuleListVO.getMinDate()
													&& null != calendarRuleListVO.getMaxDate()
													&& firstDayBetween < calendarRuleListVO.getMaxDate()) {
												point = calendarRuleListVO.getPoint();
												break;
											} else if (firstDayBetween >= calendarRuleListVO.getMinDate()
													&& null != calendarRuleListVO.getMaxDate()
													&& firstDayBetween > calendarRuleListVO.getMaxDate()) {
												continue;
											} else if (firstDayBetween >= calendarRuleListVO.getMinDate()
													&& null == calendarRuleListVO.getMaxDate()) {
												point = calendarRuleListVO.getPoint();
												break;
											} else if (firstDayBetween < calendarRuleListVO.getMinDate()) {
												point = 0;
												break;
											}
										}
									}
								}
								vo.setName(null == accountVO.getFullName() || "" == accountVO.getFullName()
										? accountVO.getName()
										: accountVO.getFullName());
								vo.setHeadPortrait(!StringUtils.isEmpty(accountVO.getHeadPortrait())?imageToBase64(accountVO.getHeadPortrait()):"");
								vo.setDay(firstDayBetween);
								vo.setIsCycle(0 != ((vo.getDay() + 1) % calendar.getCycle()) ? 0 : 1);
								vo.setIsSign(0);
								vo.setPoint(point);
								vo.setSumDays(headData.getSumDays());
								Long lastPoster = lastSign.getPosterId();
								Long posterId = 1l;
								if (lastPoster!=21) {
									posterId = lastPoster+1;
								}
								CalendarPoster poster = calendarPosterMapper.selectById(vo.getIsCycle().equals(0)?posterId:((long)new Random().nextInt(10)+22));
								if (null != poster) {
									vo.setPosterId(poster.getId());
									vo.setImgUrl(poster.getBase64Code());
									vo.setContent(poster.getContent());
								}
								// 周期第一天
							} else {
								vo.setName(null == accountVO.getFullName() || "" == accountVO.getFullName()
										? accountVO.getName()
										: accountVO.getFullName());
								vo.setHeadPortrait(!StringUtils.isEmpty(accountVO.getHeadPortrait())?imageToBase64(accountVO.getHeadPortrait()):"");
								vo.setDay(0l);
								vo.setIsSign(0);
								vo.setIsCycle(0 != ((vo.getDay() + 1) % calendar.getCycle()) ? 0 : 1);
								vo.setPoint(calendar.getFirstPoint());
								vo.setSumDays(headData.getSumDays());
								CalendarPoster poster = calendarPosterMapper.selectById(vo.getIsCycle().equals(0)?((long)new Random().nextInt(21)+1):((long)new Random().nextInt(10)+22));
								if (null != poster) {
									vo.setPosterId(poster.getId());
									vo.setImgUrl(poster.getBase64Code());
									vo.setContent(poster.getContent());
								}
							}
							// 首次签到
						} else {
							vo.setName(null == accountVO.getFullName() || "" == accountVO.getFullName()
									? accountVO.getName()
									: accountVO.getFullName());
							vo.setHeadPortrait(!StringUtils.isEmpty(accountVO.getHeadPortrait())?imageToBase64(accountVO.getHeadPortrait()):"");
							vo.setIsSign(0);
							vo.setPoint(calendar.getFirstPoint());
							vo.setDay(0l);
							vo.setIsCycle(0 != ((vo.getDay() + 1) % calendar.getCycle()) ? 0 : 1);
							vo.setSumDays(headData.getSumDays());
								CalendarPoster poster = calendarPosterMapper.selectById(vo.getIsCycle().equals(0)?((long)new Random().nextInt(21)+1):((long)new Random().nextInt(10)+22));
								if (null != poster) {
									vo.setPosterId(poster.getId());
									vo.setImgUrl(poster.getBase64Code());
									vo.setContent(poster.getContent());
								}
						}

					}
				}
				// 用户第一次签到
			} else {
				vo.setName(null == accountVO.getFullName() || "" == accountVO.getFullName() ? accountVO.getName()
						: accountVO.getFullName());
				vo.setHeadPortrait(!StringUtils.isEmpty(accountVO.getHeadPortrait())?imageToBase64(accountVO.getHeadPortrait()):"");
				vo.setDay(0l);
				vo.setIsCycle(0 != ((vo.getDay() + 1) % calendar.getCycle()) ? 0 : 1);
				vo.setIsSign(0);
				vo.setPoint(calendar.getFirstPoint());
				vo.setSumDays(0l);
				CalendarPoster poster = calendarPosterMapper.selectById(vo.getIsCycle().equals(0)?((long)new Random().nextInt(21)+1):((long)new Random().nextInt(10)+22));
				if (null != poster) {
					vo.setPosterId(poster.getId());
					vo.setImgUrl(poster.getBase64Code());
					vo.setContent(poster.getContent());
				}
			}
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			LOG.error("签到详情异常！！！" + e);
		}
		return vo;
	}

	@Override
	public HeadVO getHeadData(Long accountId, Long siteId) {
		HeadVO vo = new HeadVO();
		List<CalendarRecord> lastFirstSigns = calendarRecordMapper.getLastFirstSignByaccountId(accountId, siteId);
		List<CalendarRecord> lastSigns = calendarRecordMapper.getLastSignByaccountId(accountId, siteId);
		if (CollectionUtils.isNotEmpty(lastSigns)&&CollectionUtils.isNotEmpty(lastFirstSigns)) {
			try {
				SimpleDateFormat format = new SimpleDateFormat("yyyy-mm-dd");
				CalendarRecord lastSign = lastSigns.get(0);
				CalendarRecord lastFirstSign = lastFirstSigns.get(0);
				long consecutiveDays = (format.parse(lastSign.getSignDate()).getTime()
						- format.parse(lastFirstSign.getSignDate()).getTime()) / (60 * 60 * 24 * 1000);
				vo = calendarRecordMapper.getSumDayAndPoint(accountId, siteId);
				vo.setConsecutiveDays(consecutiveDays+1);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			vo.setConsecutiveDays(0l);
			vo.setSumDays(0l);
			vo.setPoint(0);
		}
		return vo;
	}

	@Override
	public Page<CalendarRecord> getSignDetailByAccountId(Long accountId, Long siteId, Integer type, Integer pageNo,
			Integer pageSize) {
		Page<CalendarRecord> page = new Page<CalendarRecord>();
		LOG.info("分页信息pageSize={},pageNo={}", pageSize, pageNo);
		if (pageSize != null && pageNo != null) {
			PageHelper.startPage(pageNo, pageSize);
		}
		List<CalendarRecord> list = this.baseMapper.getSignDetailByAccountId(accountId, siteId, type);
		if (pageSize != null && pageNo != null) {
			page.setTotal(PageHelper.getTotal());
			page.setSize(pageSize);
			page.setCurrent(pageNo);
			PageHelper.remove();
		}
		page.setRecords(list);
		LOG.info("签到打卡明细：" + page.getRecords());
		return page;
	}

	@Override
	public List<CalendarRecord> getSignByMonth(String month, Long accountId, Long siteId) {
		return this.baseMapper.getSignByMonth(month, accountId, siteId);
	}

	@Override
	public List<CalendarChartVO> signChart(String startDate, String endDate, Long companyId, Long siteId) {
		// 开始时间
		LocalDate startLocalDate = LocalDate.parse(startDate, DateTimeFormatter.ISO_DATE);
		// 结束时间
		LocalDate endLocalDate = LocalDate.parse(endDate, DateTimeFormatter.ISO_DATE);
		long endNum = endLocalDate.toEpochDay() - startLocalDate.toEpochDay();
		endNum = endNum <= 0 ? 1 : endNum + 1;
		List<String> retDataKey = Stream.iterate(startLocalDate, nextDate -> nextDate.plusDays(1)).limit(endNum)
				.map(item -> item.format(DateTimeFormatter.ofPattern("yyyyMMdd"))).collect(Collectors.toList());
		LOG.info("****************统计图表出来的时间信息={}", JSON.toJSONString(retDataKey));
		endDate = endDate + " 23:59:59";
		List<CalendarChartVO> list = this.baseMapper.signChart(startDate, endDate, companyId, siteId);
		Map<String, CalendarChartVO> signData = null;
		List<CalendarChartVO> listData = new ArrayList<CalendarChartVO>();
		if (!CollectionUtils.isEmpty(list)) {
			// 获取签到次数
			signData = list.stream().parallel().collect(Collectors.toMap(key -> key.getDateStr(), val -> val));
			LOG.info("****************统计图表出来的数据={}", signData.toString());
			for (String key : retDataKey) {
				CalendarChartVO obj = signData.get(key);
				if (obj == null) {
					obj = new CalendarChartVO();
					obj.setDateStr(key);
					obj.setSignNum(0);
				} else {
					obj.setDateStr(key);
				}
				listData.add(obj);
			}
		}
		return listData;
	}

	@Override
	public List<CalendarStatisticVO> list(String startDate, String endDate, String kwd, Long companyId, Long siteId) {
		endDate = endDate + " 23:59:59";
		if (StringUtils.isNotBlank(kwd)) {
			kwd = "%" + kwd + "%";
		} else {
			kwd = null;
		}
		ReportRangeAccountReqVO vo = new ReportRangeAccountReqVO();
		vo.setSiteId(siteId);
		vo.setAccountName(kwd);
		List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(vo);
		List<Long> accountIds = new ArrayList<Long>();
		Map<Long, ReportAccountRespVO> map = new HashMap<Long, ReportAccountRespVO>();
		List<CalendarStatisticVO> list = new ArrayList<CalendarStatisticVO>();
		for (ReportAccountRespVO user : accounts) {
			map.put(user.getUserId(), user);
			accountIds.add(user.getUserId());
		}
		if (!CollectionUtils.isEmpty(accountIds)) {
			list = this.baseMapper.list(startDate, endDate, companyId, siteId, accountIds);
			for (CalendarStatisticVO cVo : list) {
				ReportAccountRespVO account = map.get(cVo.getAccountId());
				cVo.setName(account.getUserName());
				cVo.setFullName(account.getUserFullName());
				cVo.setWorkNum(account.getWorkNum());
				cVo.setPosition(account.getPosition());
				cVo.setOrgName(account.getOrgName());
				List<String> orgNames = organizationClient.getChiefOrgNames(account.getOrgId(), companyId);
				String resultString = "";
				for (int i = 0; i < orgNames.size(); i++) {
					String s = orgNames.get(i);
					if (i == 0) {
						resultString += s;
					} else {
						resultString += "/" + s;
					}
				}
				cVo.setOrgNames(resultString);
			}
		}
		return list;
	}

	@Override
	public List<CalendarRecord> accountList(String startDate, String endDate, Long accountId, Long companyId,
			Long siteId) {
		endDate = endDate + " 23:59:59";
		return this.baseMapper.accountList(startDate, endDate, companyId, siteId, accountId);
	}

	@Override
	public List<CalendarDetailStatisticVO> detailList(String startDate, String endDate, Long companyId, Long siteId,
			String kwd) {
		endDate = endDate + " 23:59:59";
		if (StringUtils.isNotBlank(kwd)) {
			kwd = "%" + kwd + "%";
		} else {
			kwd = null;
		}
		ReportRangeAccountReqVO vo = new ReportRangeAccountReqVO();
		vo.setSiteId(siteId);
		vo.setAccountName(kwd);
		List<ReportAccountRespVO> accounts = reportClient.getRangeAccounts(vo);
		List<Long> accountIds = new ArrayList<Long>();
		Map<Long, ReportAccountRespVO> map = new HashMap<Long, ReportAccountRespVO>();
		for (ReportAccountRespVO user : accounts) {
			map.put(user.getUserId(), user);
			accountIds.add(user.getUserId());
		}
		List<CalendarDetailStatisticVO> list = this.baseMapper.detailList(startDate, endDate, companyId, siteId,
				accountIds);
		for (CalendarDetailStatisticVO record : list) {
			ReportAccountRespVO account = map.get(record.getAccountId());
			record.setName(account.getUserName());
			record.setFullName(account.getUserFullName());
			record.setWorkNum(account.getWorkNum());
			record.setPosition(account.getPosition());
			record.setOrgName(account.getOrgName());
			List<String> orgNames = organizationClient.getChiefOrgNames(account.getOrgId(), companyId);
			String resultString = "";
			for (int i = 0; i < orgNames.size(); i++) {
				String s = orgNames.get(i);
				if (i == 0) {
					resultString += s;
				} else {
					resultString += "/" + s;
				}
			}
			record.setOrgNames(resultString);
		}
		return list;
	}

	public static String imageToBase64(String imagePath) {
        Long t1=System.currentTimeMillis();
     	ByteArrayOutputStream data = new ByteArrayOutputStream();
         try {
             // 创建URL
             URL url = new URL(imagePath);
             byte[] by = new byte[1024];
             // 创建链接
             HttpURLConnection conn = (HttpURLConnection) url.openConnection();
             conn.setRequestMethod("GET");
             conn.setConnectTimeout(5000);
             InputStream is = conn.getInputStream();
             // 将内容读取内存中
             int len = -1;
             while ((len = is.read(by)) != -1) {
                 data.write(by, 0, len);
             }
             // 关闭流
             is.close();
         } catch (IOException e) {
             e.printStackTrace();
         }
         // 对字节数组Base64编码
         Base64Encoder encoder = new Base64Encoder();
         String encode = encoder.encode(data.toByteArray());
 		 Long t2=System.currentTimeMillis();
 		 LOG.info("图片转base64处理时间ms:"+(t2-t1));
 		return "data:image/png;base64,"+encode;
	}

	@Override
	public Map<Long, Long> signNum(List<Long> siteIds, String startDate, String endDate) {
		Map<Long, Long> map = new HashMap<Long, Long>();
		List<SignNumVO> signNum = this.baseMapper.signNum(siteIds, startDate, endDate);
		for(SignNumVO vo:signNum) {
			map.put(vo.getAccountId(), vo.getSignNum());
		}
		return map;
	}
}
