package com.yizhi.calendar.application.task;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.yizhi.calendar.application.domain.CalendarRecord;
import com.yizhi.calendar.application.service.ICalendarRecordService;

@Component
public class CalendarAccountExportAsync extends AbstractDefaultTask<String, Map<String,Object>> {

	private static final Logger logger = LoggerFactory.getLogger(CalendarAccountExportAsync.class);

	@Autowired
	private ICalendarRecordService calendarRecordService;

	@Override
	protected String execute(Map<String, Object> arg0) {
		Long accountId=(Long) arg0.get("accountId");
		Long siteId=(Long) arg0.get("siteId");
		Long companyId=(Long) arg0.get("companyId");
		Long taskId=(Long) arg0.get("taskId");
		Date submitTime=(Date) arg0.get("submitTime");
		String serialNo=(String) arg0.get("serialNo");
		String taskName=(String) arg0.get("taskName");
		String startDate = (String) arg0.get("startDate");
		String endDate = (String) arg0.get("endDate");
		Long userId = (Long) arg0.get("userId");
		String accountName = (String) arg0.get("accountName");

		/**
		 * 走异步任务
		 */
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
		working(taskContext);		
		String upLoadUrl=null;
		String requestPath= FileConstant.SAVE_PATH;
		File fileDir=new File(requestPath);
		if(!fileDir.exists()) {
			fileDir.mkdir();
		}	
		List<CalendarRecord> list = calendarRecordService.accountList(startDate, endDate, userId, companyId, siteId);
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat ff = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");		
		SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmss"); 	 	
		try {
			//excel生成过程: excel-->sheet-->row-->cell
			// 第一步，创建一个Excel文件
			XSSFWorkbook wb = new XSSFWorkbook();
			// 第二步，在webbook中添加一个sheet,对应Excel文件中的sheet
			XSSFSheet sheet = wb.createSheet("打卡记录");
			sheet.setDefaultColumnWidth(26);
			XSSFFont font = wb.createFont();
			font.setFontName("新宋体");
			font.setFontHeightInPoints((short) 16);
			font.setBold(true);
			XSSFCellStyle style = wb.createCellStyle();
			style.setFont(font);
			style.setAlignment(HorizontalAlignment.LEFT); // 创建一个居中格式
			style.setVerticalAlignment(VerticalAlignment.CENTER);
			// 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
			XSSFRow row = sheet.createRow((int) 0);
			row.setHeightInPoints((float) 20.25);
			// 合并单元格  
			CellRangeAddress cra=new CellRangeAddress(0,0,0,2); // 起始行, 终止行, 起始列, 终止列  
			sheet.addMergedRegion(cra);
			XSSFCell cell = row.createCell((short) 0);
			cell.setCellValue(accountName+"打卡记录表 统计日期："+startDate+" 至 "+endDate);
			cell.setCellStyle(style);
			XSSFRow row1 = sheet.createRow((int) 1);
			row1.setHeightInPoints((float) 18.75);
			// 合并单元格  
			CellRangeAddress cra1=new CellRangeAddress(1,1,0,2); // 起始行, 终止行, 起始列, 终止列  
			sheet.addMergedRegion(cra1);
			XSSFCell cell1 = row1.createCell((short) 0);
			cell1.setCellValue("报表生成时间："+ff.format(submitTime));
			XSSFFont font1 = wb.createFont();
			font1.setFontName("新宋体");
			font1.setFontHeightInPoints((short) 14);
			font1.setBold(false);
			XSSFCellStyle style1 = wb.createCellStyle();
			style1.setFont(font1);
			style1.setAlignment(HorizontalAlignment.LEFT); // 创建一个居中格式
			style1.setVerticalAlignment(VerticalAlignment.CENTER);
			cell1.setCellStyle(style1);
			XSSFRow row2 = sheet.createRow((int) 2);
			XSSFFont font2 = wb.createFont();
			XSSFCellStyle style2 = wb.createCellStyle();
			font2.setFontName("新宋体");
			font2.setFontHeightInPoints((short) 12);
			font2.setBold(true);
			style2.setFont(font2);
			style2.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
			style2.setVerticalAlignment(VerticalAlignment.CENTER);
			row2.setRowStyle(style2);
			row2.setHeightInPoints((float) 14.25);
			row2.createCell((short) 0).setCellValue("打卡日期");
			row2.getCell(0).setCellStyle(style2);
			row2.createCell((short) 1).setCellValue("打卡时间"); 
			row2.getCell(1).setCellStyle(style2);
			row2.createCell((short) 2).setCellValue("获得积分"); 
			row2.getCell(2).setCellStyle(style2);
			CalendarRecord vo = new CalendarRecord();
			for(int i=0;i<list.size();i++) {
				XSSFFont font3 = wb.createFont();
				XSSFCellStyle style3 = wb.createCellStyle();
					font3.setBold(false);
					font3.setFontName("宋体");
					font3.setFontHeightInPoints((short) 11);
					style3.setFont(font3);
					style3.setAlignment(HorizontalAlignment.LEFT); // 创建一个居中格式
					style3.setVerticalAlignment(VerticalAlignment.CENTER);
					row = sheet.createRow(i + 3);
					row.setRowStyle(style3);
					row.setHeightInPoints((float) 13.5);
					// 第四步，创建单元格，并设置值
					vo=list.get(i);
					if(vo!=null&&vo.getSignDate()!=null) {
						row.createCell((short) 0).setCellValue(vo.getSignDate());
						row.getCell(0).setCellStyle(style3);
					}
					if(vo!=null&&vo.getSignTime()!=null) {
						row.createCell((short) 1).setCellValue(vo.getSignTime());
						row.getCell(1).setCellStyle(style3);
					}				
					if(vo!=null&&vo.getPoint()!=null) {
						row.createCell((short) 2).setCellValue(vo.getPoint());
						row.getCell(2).setCellStyle(style3);
					}
				}
				
			// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
			StringBuffer fileNameSb = new StringBuffer().append(accountName+"打卡记录表").append(sf.format(submitTime)).append(".xlsx");
			String fileName = fileNameSb.toString();
			String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
			FileOutputStream os=null;
			File file=null;
			try {
				os= new FileOutputStream(path);
				wb.write(os);
				//阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file=new File(path);
				success(taskContext,"成功", upLoadUrl);
			} catch (Exception e1) {
				logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误",e1);	
				fail(taskContext, "写入数据到Excel的过程中或者上传到阿里云中发生错误"+e1.getMessage());
			}
			finally {
				if(os!=null) {
				   os.close();
				}
				if(wb!=null) {
				   wb.close();
				}
				if(file!=null) {
				   file.delete();
				}
			}
		} catch (Exception e) {
			logger.error(accountName+"打卡记录导出过程中发生错误，请查看日志",e);
			fail(taskContext, accountName+"打卡记录导出过程中发生错误，请查看日志"+e.getMessage());	
		}
		return upLoadUrl;
	}

}
